<?php
/*
Plugin Name: SVN Auto Upgrade
Description: Hook into plugin and core upgrader to support SVN driven sites. Now you can freely use the WordPress auto upgrade features without worrying about the Subversion impact. Just upgrade in WordPress and then commit the changes in SVN.
Author: Shane & Peter, Inc.
Version: 1.0
Author URI: http://www.shaneandpeter.com
*/

if ( !class_exists( 'SVNAutoUpgrader' ) ) {
	class SVNAutoUpgrader {
		
		public function __construct() {
			$this->addFilters();
			$this->addActions();
		}
		
		public function addFilters() {
			add_filter('upgrader_pre_install', array(&$this, 'make_svn_backup'), 10, 2);
			add_filter('upgrader_post_install', array(&$this, 'restore_svn_folders'), 10, 3);
		}
		
		public function addActions() {
			add_action('admin_footer', array(&$this, 'global_svn_add_remove'));
		}

		public function make_svn_backup( $return, $targets) {
			global $wp_filesystem;
			// copy folder to maintain svn structure
			if ( isset( $targets['plugin'] ) ) {
				$plugin_dir = $wp_filesystem->wp_plugins_dir() . dirname( $targets['plugin'] );
				if ( $this->svn_test( $plugin_dir ) ) {
					$this->recurse_svn_copy( $plugin_dir, $plugin_dir.'.temp' );
				}
			}
			return $return;
		}

		public function restore_svn_folders( $return, $targets, $results ) {
			// restore svn structure
			global $wp_filesystem;
			// copy folder to maintain svn structure
			if ( isset( $targets['plugin'] ) ) {
				$plugin_dir = $wp_filesystem->wp_plugins_dir() . dirname( $targets['plugin'] );
				if ( is_dir( $plugin_dir.'.temp' ) && $this->svn_test( $plugin_dir.'.temp' ) ) {
					$this->recurse_svn_copy( $plugin_dir.'.temp', $plugin_dir );
					$wp_filesystem->rmdir( $plugin_dir.'.temp', true );
					$this->svn_add_remove( $plugin_dir );
				}
			}
			return $return;
		}

		public function recurse_svn_copy( $src, $dst, $in_svn = false ) {
			if (!is_dir($dst)) { @mkdir($dst); }
		    $dir = opendir($src);
		    while(false !== ( $file = readdir($dir)) ) {
		        if (( $file != '.' ) && ( $file != '..' ) ) {
					if ( $file == '.svn' ) {
		                $this->recurse_svn_copy($src . '/' . $file, $dst . '/' . $file, true);						
					} elseif ( is_dir($src . '/' . $file) ) {
		                $this->recurse_svn_copy($src . '/' . $file, $dst . '/' . $file, true);
		            } elseif ($in_svn) {
		                copy($src . '/' . $file,$dst . '/' . $file);
		            }
		        }
		    }
		    closedir($dir);
		}
		
		public function global_svn_add_remove() {
			$action = isset($_GET['action']) ? $_GET['action'] : 'upgrade-core';
			if ( 'do-core-upgrade' == $action ) {
				check_admin_referer('upgrade-core');
				$this->svn_add_remove( ABSPATH );
			}
		}
		
		public function svn_add_remove( $path ) {
			if ($this->svn_test($path)) {
				shell_exec("svn add `svn status $path | grep '^?' | awk '{ print $2 }' | xargs`");
				shell_exec("svn delete --force `svn status $path | grep '^!' | awk '{ print $2 }' | xargs`");
			}
		}
		
		public function svn_test( $path ) {
			return is_dir( trailingslashit($path) . '.svn' );
		}

	}

	global $SVNAutoUpgrader;
	$SVNAutoUpgrader = new SVNAutoUpgrader();
}
?>